from django.contrib.auth import get_user_model

from rules import predicate

from aleksis.core.models import Group, Person
from aleksis.core.util.predicates import check_object_permission

from .models import EventRegistration, Voucher

User = get_user_model()


@predicate
def is_own_voucher(user: User, obj: Voucher) -> bool:
    """Predicate which checks if the voucher belongs to the user."""
    return obj.person == user.person


@predicate
def is_own_registration(user: User, registration: EventRegistration) -> bool:
    """Predicate which checks if the registration belongs to the user."""
    return obj.person == user.person


@predicate
def is_organiser(user: User, obj: EventRegistration) -> bool:
    """Predicate which checks if the user is an organiser."""
    return user.person in obj.event.linked_group.owners.all()

@predicate
def is_event_published(obj: EventRegistration) -> bool:
    """Predicate which checks if the event is published."""
    return obj.published
